using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using TTDotNet.Util;
using TTDotNet.Engine.ThreeDEngine;
using Microsoft.DirectX;

namespace TTDotNet.Engine
{
    public class Building : GameObject
    {
        protected bool isBuilt;
        protected BuildingInfo buildingInfo;
        protected List<Coordinate> areaCovered;

        public Building(Vector3 location) : base(location)
        {
            
        }

        protected void SetBuildingArea(List<Coordinate> area)
        {
            areaCovered = area;
        }

        protected void SetupMesh(Coordinate coord, RotateType rotType, ObjectType type)
        {
            float meshX = -1;
            float meshZ = -1;

            if (rotType == RotateType.Zero || rotType == RotateType.OneEighty)
            {
                meshX = coord.X + buildingInfo.Dimensions.Width / 2.0f - 0.5f;
                meshZ = coord.Z + buildingInfo.Dimensions.Height / 2.0f - 0.5f;
            }
            else
            {
                meshX = coord.X + buildingInfo.Dimensions.Height / 2.0f - 0.5f;
                meshZ = coord.Z + buildingInfo.Dimensions.Width / 2.0f - 0.5f;
            }

            Vector3 rotation = new Vector3(0, 0, 0);
            if (rotType == RotateType.Ninety) rotation.Y = 90;
            else if (rotType == RotateType.OneEighty) rotation.Y = 180;
            else if (rotType == RotateType.TwoSeventy) rotation.Y = 270;

            InstancedMesh iMesh = Game.meshManager.CreateMesh(buildingInfo.MeshPath,
                                                                new Vector3(meshX, (Game.gameWorld.Map.GetTileMaxHeight(coord.X, coord.Z)) / Game.heightScale, meshZ), rotation, type);

            SetMesh(iMesh);
        }
   
        protected bool NotifyTiles()
        {
            bool result = true;
            
            Size dimensions = buildingInfo.Dimensions;

            for (int z = 0; z < dimensions.Height; z++)
            {
                for (int x = 0; x < dimensions.Width; x++)
                {
                    if (buildingInfo.SquareOccupied(x, z))
                    {
                        Coordinate coord = new Coordinate(x + (int)Location.X, z + (int)Location.Z);
                        result = result && Game.gameWorld.Map.AddGameObject(coord, this);
                    }
                }
            }

            isBuilt = result;
            return result;

        }

        public bool IsBuilt
        {
            get { return isBuilt; }
        }
    }

    public struct BuildingInfo
    {
        private string meshPath;
        private Size size;
        private bool[,] occupied;

        public BuildingInfo(string meshPath, Size size, BitArray squares)
        {
            if (size.Height * size.Width == squares.Count)
            {
                occupied = new bool[size.Width, size.Height];

                for (int z = 0; z < size.Height; z++)
                {
                    for (int x = 0; x < size.Width; x++)
                    {
                        occupied[x, z] = squares[z * size.Width + x];
                    }
                }

                this.size = size;
                this.meshPath = meshPath;
            }
            else
            {
                throw new Exception("Invalid occupied squares mask");
            }
        }

        public string MeshPath
        {
            get
            {
                return meshPath;
            }
        }

        public Size Dimensions
        {
            get
            {
                return size;
            }
        }

        public bool SquareOccupied(Coordinate coord)
        {
            return occupied[coord.X, coord.Z];
        }

        public bool SquareOccupied(int x, int z)
        {
            return occupied[x, z];
        }

    }
}
