using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;

using TTDotNet.Engine.GUI.Windows;
using TTDotNet.Util;
using TTDotNet.Engine.ThreeDEngine;
using TTDotNet.Engine.Companies;
using Microsoft.DirectX.Direct3D;

namespace TTDotNet.Engine.GUI
{
    public class ScreenStatusBar : Window
    {
        private float height = 24;
        private const float centreWidth = 400;
        private const float dateWidth = 200;
        private const float border = 2;
        private Company company;

        public ScreenStatusBar(Company company, PointF screenLocation)
        {
            this.company = company;
            float screenWidth = (2 * dateWidth) + centreWidth + (2 * border);

            SetScreenRectangle(new RectangleF(screenLocation, new SizeF(screenWidth, height)));
            SetupButtons();
            AlwaysOnTop = true;
        }

        private void SetupButtons()
        {
            SetupRoadConc();
        }

        private void SetupRoadConc()
        {
            int controlNum = 0;

            float vWidth = Game.device.Viewport.Width;
            float vHeight = Game.device.Viewport.Height;

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(0, 0), new SizeF(screenRectangle.Width, screenRectangle.Height)), "", 1, Color.Black, "Arial",
                                            Game.textureCacher.LoadTexture("artwork/colours/black.jpg"), DrawTextFormat.None);

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(0, 0), new SizeF(dateWidth, height)), "",
                                                        14, Color.White, "Arial", Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), DrawTextFormat.Center);

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(dateWidth + border, 0), new SizeF(centreWidth, height)), "",
                                                        14, Color.White, "Arial", Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), DrawTextFormat.Center);

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(dateWidth + centreWidth + (2 * border), 0), new SizeF(dateWidth, height)), "",
                                                        14, Color.White, "Arial", Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), DrawTextFormat.Center);
        }

        private void CreateRoadConcWindow(object data, object tag)
        {
            Game.newGUI.OpenNewWindow(new RoadConcWindow(new PointF(300, 300)));
        }

        public override void Update()
        {
            screenRectangle.X = (Game.device.Viewport.Width /2) - (screenRectangle.Width / 2);
            screenRectangle.Y = Game.device.Viewport.Height - 24;
            windowControls[1].Text = Game.gameWorld.CurrentDate.ToLongDateString();
            windowControls[2].Text = company.Name;
            windowControls[3].Text = "" + company.BankBalance.ToString();

            if (company.BankBalance < 0) windowControls[3].FontColor = Color.Pink;
            else windowControls[3].FontColor = Color.White;
        }

    }
}
