using System;
using System.Drawing;
using System.Collections.Generic;
using TTDotNet.Engine.Stations.Depot;
using TTDotNet.Engine.GUI.Windows;
using TTDotNet.Engine.GUI.Buttons;
using TTDotNet.Engine.Vehicles;
using TTDotNet.Engine.Stations.Depot;
using TTDotNet.Util;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX;

namespace TTDotNet.Engine.GUI.Windows.Depot.Vehicles
{
    public class NewVehiclesWindow : Window
    {
        private int cSize = 24;
        private int border = 2;
        private int vehicleRows = 10;
        private int vehicleWidth = 400;
        private int vehicleHeight = 25;
        private int summaryLineHeight = 25;
        private int summaryLines = 5;
        private int listboxNum;
        private VehicleType type;
        private int summaryControlStart;
        private RadioGroup radioGroup;
        private ListBox vehicleLB;
        private Stations.Depot.Depot depot;

        public NewVehiclesWindow(PointF screenLocation, VehicleType vehcType, Stations.Depot.Depot depot)
        {
            this.depot = depot;
            this.type = vehcType;

            if(vehcType == VehicleType.Rail) summaryLines = 6;

            int width = vehicleWidth + (2* border);
            int height = (2 * cSize) + (vehicleRows * vehicleHeight) + ((vehicleRows * 2 + 5) * border) + (summaryLineHeight * summaryLines);

            SetScreenRectangle(new RectangleF(screenLocation, new SizeF(width, height)));

            int controlNum = 0;

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(0, 0), new SizeF(width, height)), "", 1, Color.Black, "Arial",
                                            Game.textureCacher.LoadTexture("artwork/colours/dark_spring_green.jpg"), DrawTextFormat.Center | DrawTextFormat.VerticalCenter);

            //Close button. 
            windowControls[controlNum++] = new CloseButton(new RectangleF(new PointF(border, border), new SizeF(cSize, cSize)),
                                            Game.textureCacher.LoadTexture("artwork/128/close.jpg"), Game.textureCacher.LoadTexture("artwork/128/closeDown.jpg"), this);

            windowControls[controlNum++] = new TitleBar(new RectangleF(new PointF(cSize + (2 * border), border), new SizeF(width - (cSize + (3 * border)), cSize)),
                                            "New Road Vehicles", 14, Color.White, "Arial", Game.textureCacher.LoadTexture("artwork/colours/ttdOrange.jpg"), this);

            windowControls[controlNum++] = new ListBox(new RectangleF(new PointF(border, cSize + (2 * border)), new SizeF(width - (2*border), vehicleRows * (vehicleHeight + (2 * border)))),
                                            18, "", 1, Color.Black, "Arial", vehicleHeight + (border * 2), Game.textureCacher.LoadTexture("artwork/colours/black.jpg"));
            vehicleLB = windowControls[controlNum - 1] as ListBox;
            vehicleLB.OnSelectionChanged += new ListBox.ListBoxDelegate(NewVehiclesWindow_OnSelectionChanged);

            radioGroup = new RadioGroup();

            
            LoadVehicles();

            summaryControlStart = controlNum;

            float summaryX = border;
            float summaryY = (3 * border) + cSize + vehicleLB.Size.Height;

            windowControls[controlNum++] = new Label(new RectangleF(new PointF(summaryX, summaryY), new SizeF(width - (2*border), summaryLineHeight * summaryLines)), "", 14, Color.White, "Arial",
                                            Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), DrawTextFormat.Left);
            //windowControls[summaryControlStart].MultiLine = true;

            SetupSummary();

            float bWidth = (width - cSize - (5 * border)) / 2;

            windowControls[controlNum++] = new GameButton(new RectangleF(new PointF(border, height - cSize - border), new SizeF(bWidth, cSize)), "Build Vehicle", 14, Color.Black, "Arial",
                                            Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), Game.textureCacher.LoadTexture("artwork/colours/silver.jpg"));
            windowControls[controlNum - 1].OnControl += new Control.ControlDelegate(BuildVehicle);

            windowControls[controlNum++] = new GameButton(new RectangleF(new PointF((2 * border) + bWidth, height - cSize - border), new SizeF(bWidth, cSize)), "Rename", 14, Color.Black, "Arial",
                                            Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), Game.textureCacher.LoadTexture("artwork/colours/silver.jpg"));
        }

        void NewVehiclesWindow_OnSelectionChanged()
        {
            radioGroup.Select(vehicleLB.SelectedItem);
            SetupSummary();
        }

        private void LoadVehicles()
        {
            radioGroup.Clear();

            int rbNum = 0;

            List<VehicleInfo> infos = RoadVehicle.VehicleInfos;

            for (int row = 0; row < infos.Count; row++)
            {
                if (row < infos.Count)
                {
                    ListableItem listItem = vehicleLB.AddItem();
                    RadioButton rb = new RadioButton(1, new RectangleF(border, border, vehicleWidth - cSize, vehicleHeight),
                                                infos[row].ModelName, 14, Color.Black, "Arial", false, true, Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"), Game.textureCacher.LoadTexture("artwork/colours/ttdGrey.jpg"));
                    rb.SetSelectedFontColour(Color.White);
                    radioGroup.Add(rbNum++, rb);
                    listItem.AddControl(rb);

                    if (row == 0) rb.Select();
                }
            }
        }

        private void SetupSummary()
        {
            int controlNum = summaryControlStart;

            float top = this.Area.Height - cSize - (2 * border) - (summaryLines * summaryLineHeight);
            float width = this.Area.Width;

            List<VehicleInfo> infos = RoadVehicle.VehicleInfos;

            int vn = vehicleLB.SelectedItem;

            string summary = "Cost: " + infos[vn].Cost + " Speed: " + infos[vn].MaxSpeed + " mph";
            summary += "\nRunning Cost: " + infos[vn].MaintenanceCost + "/yr";
            summary += "\nCapacity: " + infos[vn].CapacityInfo;
            summary += "\nDesigned: " + (infos[vn].YearInvented.Year - 1) + " Life: " + infos[vn].ModelLife + " years";
            summary += "\nMax. Reliability: " + infos[vn].MaxReliability + "%";
            windowControls[controlNum].Text = summary;

        }

        private void BuildVehicle(object data, object tag)
        {
            if (Game.gameWorld.GetCompany(Game.playerNumber).RemoveFunds(RoadVehicle.VehicleInfos[vehicleLB.SelectedItem].Cost, false))
            {

                RoadVehicle myVehc = new RoadVehicle("My First", RoadVehicle.VehicleInfos[vehicleLB.SelectedItem], Game.gameWorld.GetCompany(Game.playerNumber), new DateTime(), new Vector3(depot.Location.X, depot.Location.Y, depot.Location.Z));

                Game.gameWorld.AddVehicle(0, myVehc);

            }
            else
            {

            }
        }

    }
}