//--------------------------------------------------------------------------------------
// File: ManagedInstancing.fx
//
// The effect file for the ManagedInstancing sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float4x4 WorldViewProjection : WORLD : register(c0);

texture Tex0;

sampler Tex0Sampler=
sampler_state
{
    Texture = <Tex0>;
    
    MinFilter =anisotropic;
    MagFilter =anisotropic;
    MipFilter =anisotropic;
    AddressU = mirror;
    AddressV = mirror;
    MaxAnisotropy = 16;
};

float3 InstanceData[240] : register(c16);

//application to vertex structure
struct a2v
{ 
    float4 position   : POSITION;
    float3 normal	  : TEXCOORD2;
    float2 tex0       : TEXCOORD0;    
    float4 instance    : TEXCOORD1;
};
 
//vertex to pixel shader structure
struct v2p
{   	
	float4 position    : POSITION;
	float2 tex0        : TEXCOORD0;
};
 
//pixel shader to screen
struct p2f
{
    float4 color    : COLOR0;
};


void ps( in v2p IN,  out p2f OUT)
{		
    OUT.color = tex2D(Tex0Sampler, IN.tex0); 
};

void vs( in a2v IN, out v2p OUT )
{       
	float3 data = InstanceData[ IN.instance.x ];	
	
	float4 translatedPos = IN.position;
	
	translatedPos.x += data.x;
	translatedPos.y += data.y;
	translatedPos.z += data.z;
	
	OUT.position = mul(translatedPos, WorldViewProjection);
	OUT.tex0 = IN.tex0;
};


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique BatchedShaderInstancing
{
    pass P0
    {      
        VertexShader = compile vs_2_0 vs();           
        PixelShader  = compile ps_2_0 ps();          
    }
}
