//--------------------------------------------------------------------------------------
// File: ManagedInstancing.fx
//
// The effect file for the ManagedInstancing sample.  
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
float4x4 WorldViewProjection;
float4x4 RotationMatrixX;
float4x4 RotationMatrixY;
float4x4 RotationMatrixZ;

texture Tex0;
sampler Tex0Sampler=
sampler_state
{
    Texture = <Tex0>;
    
    MinFilter = Anisotropic;
    MagFilter = Anisotropic;
    MipFilter = Anisotropic;    
};


//application to vertex structure
struct a2v
{ 
    float4 position     : POSITION0;
    float3 normal	    : NORMAL;
    float2 tex0         : TEXCOORD0; 
    float4 instancePos  : COLOR0;
    float4 instanceRot  : DEPTH;
};
 
//vertex to pixel shader structure
struct v2p
{   	
	float4 position    : POSITION0;
    float2 tex0        : TEXCOORD0;
};
 
//pixel shader to screen
struct p2f
{
    float4 color    : COLOR0;
};

void ps( in v2p IN,  out p2f OUT)
{	
    OUT.color = tex2D(Tex0Sampler, IN.tex0); 
};

void vs( in a2v IN, out v2p OUT )
{      
	float4 data = IN.instancePos;	
	float4 instancePos = IN.position;	
	instancePos.x += data.x;
	instancePos.y += data.y;
	instancePos.z += data.z;
	
	float4 rotData = IN.instanceRot;
		
	RotationMatrixX[0].x = 1;
	RotationMatrixX[1].y = 1;
	RotationMatrixX[2].z = 1;
	RotationMatrixX[3].w = 1;
	RotationMatrixY[0].x = 1;
	RotationMatrixY[1].y = 1;
	RotationMatrixY[2].z = 1;
	RotationMatrixY[3].w = 1;
	RotationMatrixZ[0].x = 1;
	RotationMatrixZ[1].y = 1;
	RotationMatrixZ[2].z = 1;
	RotationMatrixZ[3].w = 1;
	
	RotationMatrixX[1].y = cos(rotData.x);
	RotationMatrixX[1].z = -sin(rotData.x);
	RotationMatrixX[2].y = sin(rotData.x);
	RotationMatrixX[2].z = cos(rotData.x);
	
	RotationMatrixY[0].x = cos(rotData.y);
	RotationMatrixY[0].z = -sin(rotData.y);
	RotationMatrixY[2].x = sin(rotData.y);
	RotationMatrixY[2].z = cos(rotData.y);
	
	RotationMatrixZ[0].x *= cos(rotData.z);
	RotationMatrixZ[0].y = -sin(rotData.z);
	RotationMatrixZ[1].x = sin(rotData.z);
	RotationMatrixZ[1].y *= cos(rotData.z);
	
	OUT.position = mul(IN.position, RotationMatrixX);   
	OUT.position = mul(OUT.position, RotationMatrixY);   		
	OUT.position = mul(OUT.position, RotationMatrixZ);   	
	OUT.position.x += data.x;
	OUT.position.y += data.y;
	OUT.position.z += data.z;
	OUT.position = mul(OUT.position, WorldViewProjection);
	OUT.tex0 = IN.tex0;
};


//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique BatchedShaderInstancing
{
    pass P0
    {      
        VertexShader = compile vs_2_0 vs();           
        PixelShader  = compile ps_2_0 ps();          
    }
}
